@@ -10,6 +10,7 @@ __PACKAGE__->load_components(qw/
   Serialize::Storable
   InflateColumn
   Relationship
+  PK::Auto
   PK
   Row
   ResultSourceProxy::Table
@@ -17,7 +18,7 @@ __PACKAGE__->load_components(qw/
 
 1;
 
-=head1 NAME 
+=head1 NAME
 
 DBIx::Class::Core - Core set of DBIx::Class modules
 
@@ -35,6 +36,8 @@ The core modules currently are:
 
 =over 4
 
+=item L<DBIx::Class::Serialize::Storable>
+
 =item L<DBIx::Class::InflateColumn>
 
 =item L<DBIx::Class::Relationship>
@@ -49,10 +52,6 @@ The core modules currently are:
 
 =back
 
-If you are also using a L<DBIx::Class::PK::Auto> component, please
-make sure you load it correctly. Refer to
-L<DBIx::Class::PK::Auto/DESCRIPTION> for more information.
-
 =head1 AUTHORS
 
 Matt S. Trout <mst@shadowcatsystems.co.uk>
