@@ -1,5 +1,8 @@
 package DBIx::Class::DB;
 
+use strict;
+use warnings;
+
 use base qw/DBIx::Class/;
 use DBIx::Class::Schema;
 use DBIx::Class::Storage::DBI;
@@ -8,8 +11,11 @@ use DBI;
 
 __PACKAGE__->load_components(qw/ResultSetProxy/);
 
-*dbi_commit = \&txn_commit;
-*dbi_rollback = \&txn_rollback;
+{
+    no warnings 'once';
+    *dbi_commit = \&txn_commit;
+    *dbi_rollback = \&txn_rollback;
+}
 
 sub storage { shift->schema_instance(@_)->storage; }
 
@@ -23,9 +29,9 @@ sub resultset_instance {
   return $source->resultset;
 }
 
-=head1 NAME 
+=head1 NAME
 
-DBIx::Class::DB - Non-recommended classdata schema component
+DBIx::Class::DB - (DEPRECATED) classdata schema component
 
 =head1 SYNOPSIS
 
@@ -39,7 +45,8 @@ DBIx::Class::DB - Non-recommended classdata schema component
   package MyDB::MyTable;
 
   use base qw/MyDB/;
-  __PACKAGE__->load_components('Core'); # just load this in MyDB if it will always be there
+  __PACKAGE__->load_components('Core'); # just load this in MyDB if it will
+                                        # always be there
 
   ...
 
@@ -47,8 +54,8 @@ DBIx::Class::DB - Non-recommended classdata schema component
 
 This class is designed to support the Class::DBI connection-as-classdata style
 for DBIx::Class. You are *strongly* recommended to use a DBIx::Class::Schema
-instead; DBIx::Class::DB will continue to be supported but new development
-will be focused on Schema-based DBIx::Class setups.
+instead; DBIx::Class::DB will not undergo new development and will be moved
+to being a CDBICompat-only component before 1.0.
 
 =head1 METHODS
 
@@ -60,7 +67,7 @@ Sets or gets the storage backend. Defaults to L<DBIx::Class::Storage::DBI>.
 
 ****DEPRECATED****
 
-Sets or gets the class to use for resolving a class. Defaults to 
+Sets or gets the class to use for resolving a class. Defaults to
 L<DBIx::Class::ClassResolver::Passthrough>, which returns whatever you give
 it. See resolve_class below.
 
@@ -128,7 +135,7 @@ sub txn_rollback { shift->schema_instance->txn_rollback(@_); }
 
 Executes a block of code transactionally. If this code reference
 throws an exception, the transaction is rolled back and the exception
-is rethrown. See txn_do in L<DBIx::Class::Schema> for more details.
+is rethrown. See L<DBIx::Class::Schema/"txn_do"> for more details.
 
 =cut
 
