@@ -5,11 +5,7 @@ use warnings;
 
 sub belongs_to {
   my ($class, $rel, $f_class, $cond, $attrs) = @_;
-  eval "require $f_class";
-  if ($@) {
-    $class->throw_exception($@) unless $@ =~ /Can't locate/;
-  }
-  
+  $class->ensure_class_loaded($f_class);
   # no join condition or just a column name
   if (!ref $cond) {
     my %f_primaries = map { $_ => 1 } eval { $f_class->primary_columns };
@@ -18,9 +14,9 @@ sub belongs_to {
 
     my ($pri, $too_many) = keys %f_primaries;
     $class->throw_exception("Can't infer join condition for ${rel} on ${class}; ${f_class} has no primary keys")
-      unless defined $pri;      
+      unless defined $pri;
     $class->throw_exception("Can't infer join condition for ${rel} on ${class}; ${f_class} has multiple primary keys")
-      if $too_many;      
+      if $too_many;
 
     my $fk = defined $cond ? $cond : $rel;
     $class->throw_exception("Can't infer join condition for ${rel} on ${class}; $fk is not a column")
