@@ -10,7 +10,7 @@ __PACKAGE__->load_components(qw/AccessorGroup/);
 
 __PACKAGE__->mk_group_accessors('simple' => 'result_source');
 
-=head1 NAME 
+=head1 NAME
 
 DBIx::Class::Row - Basic row methods
 
@@ -36,10 +36,12 @@ sub new {
   $class = ref $class if ref $class;
   my $new = bless { _column_data => {} }, $class;
   if ($attrs) {
-    $new->throw_exception("attrs must be a hashref") unless ref($attrs) eq 'HASH';
-    while (my ($k, $v) = each %$attrs) {
-      $new->throw_exception("No such column $k on $class") unless $class->has_column($k);
-      $new->store_column($k => $v);
+    $new->throw_exception("attrs must be a hashref")
+      unless ref($attrs) eq 'HASH';
+    foreach my $k (keys %$attrs) {
+      $new->throw_exception("No such column $k on $class")
+        unless $class->has_column($k);
+      $new->store_column($k => $attrs->{$k});
     }
   }
   return $new;
@@ -120,9 +122,9 @@ sub update {
 
   $obj->delete
 
-Deletes the object from the database. The object is still perfectly usable
-accessor-wise etc. but ->in_storage will now return 0 and the object must
-be re ->insert'ed before it can be ->update'ed
+Deletes the object from the database. The object is still perfectly usable,
+but ->in_storage() will now return 0 and the object must re inserted using
+->insert() before ->update() can be used on it.
 
 =cut
 
@@ -134,8 +136,8 @@ sub delete {
     $self->throw_exception("Cannot safely delete a row in a PK-less table")
       if ! keys %$ident_cond;
     foreach my $column (keys %$ident_cond) {
-	    $self->throw_exception("Can't delete the object unless it has loaded the primary keys")
-	      unless exists $self->{_column_data}{$column};
+            $self->throw_exception("Can't delete the object unless it has loaded the primary keys")
+              unless exists $self->{_column_data}{$column};
     }
     $self->result_source->storage->delete(
       $self->result_source->from, $ident_cond);
@@ -230,8 +232,8 @@ Sets more than one column value at once.
 
 sub set_columns {
   my ($self,$data) = @_;
-  while (my ($col,$val) = each %$data) {
-    $self->set_column($col,$val);
+  foreach my $col (keys %$data) {
+    $self->set_column($col,$data->{$col});
   }
   return $self;
 }
@@ -253,6 +255,7 @@ sub copy {
       if $self->result_source->column_info($col)->{is_auto_increment};
   }
   my $new = bless { _column_data => $col_data }, ref $self;
+  $new->result_source($self->result_source);
   $new->set_columns($changes);
   $new->insert;
   foreach my $rel ($self->result_source->relationships) {
@@ -278,9 +281,9 @@ Sets a column value without marking it as dirty.
 
 sub store_column {
   my ($self, $column, $value) = @_;
-  $self->throw_exception( "No such column '${column}'" ) 
+  $self->throw_exception( "No such column '${column}'" )
     unless exists $self->{_column_data}{$column} || $self->has_column($column);
-  $self->throw_exception( "set_column called for ${column} without value" ) 
+  $self->throw_exception( "set_column called for ${column} without value" )
     if @_ < 3;
   return $self->{_column_data}{$column} = $value;
 }
@@ -310,7 +313,7 @@ sub inflate_result {
     if (ref($pre_val->[0]) eq 'ARRAY') { # multi
       my @pre_objects;
       foreach my $pre_rec (@$pre_val) {
-        unless ($pre_source->primary_columns == grep { exists $pre_rec->[0]{$_} 
+        unless ($pre_source->primary_columns == grep { exists $pre_rec->[0]{$_}
            and defined $pre_rec->[0]{$_} } $pre_source->primary_columns) {
           next;
         }
@@ -320,11 +323,11 @@ sub inflate_result {
       $new->related_resultset($pre)->set_cache(\@pre_objects);
     } elsif (defined $pre_val->[0]) {
       my $fetched;
-      unless ($pre_source->primary_columns == grep { exists $pre_val->[0]{$_} 
+      unless ($pre_source->primary_columns == grep { exists $pre_val->[0]{$_}
          and !defined $pre_val->[0]{$_} } $pre_source->primary_columns)
       {
         $fetched = $pre_source->result_class->inflate_result(
-                      $pre_source, @{$pre_val});      
+                      $pre_source, @{$pre_val});
       }
       my $accessor = $source->relationship_info($pre)->{attrs}{accessor};
       $class->throw_exception("No accessor for prefetched $pre")
@@ -357,7 +360,8 @@ sub update_or_insert {
 
 =head2 is_changed
 
-  my @changed_col_names = $obj->is_changed
+  my @changed_col_names = $obj->is_changed();
+  if ($obj->is_changed()) { ... }
 
 =cut
 
@@ -365,13 +369,28 @@ sub is_changed {
   return keys %{shift->{_dirty_columns} || {}};
 }
 
+=head2 is_column_changed
+
+  if ($obj->is_column_changed('col')) { ... }
+
+=cut
+
+sub is_column_changed {
+  my( $self, $col ) = @_;
+  return exists $self->{_dirty_columns}->{$col};
+}
+
 =head2 result_source
 
   Accessor to the ResultSource this object was created from
 
 =head2 register_column
 
-=head3 Arguments: ($column, $column_info)
+=over 4
+
+=item Arguments: $column, $column_info
+
+=back
 
   Registers a column on the class. If the column_info has an 'accessor' key,
   creates an accessor named after the value if defined; if there is no such
